/*------------------------------------------------------------------------------*/
/*		ft8_cat_40m_20m_06.ino		26.02.2022			*/
/*------------------------------------------------------------------------------*/
/*

	Frequenzen fuer das 40m Band =  7.074MHz, plus ZF, nur ZF
	Frequenzen fuer das 20m Band = 14.074MHz, plus ZF, nur ZF
	Nun per Jumper umschaltbar, an Pin A0 = PC0 angschlossen.
		offen	ist 40m
		zu	ist 20m

	Mein Prinzip: Eine ganze Schwingung/Periode zweimal messen
	und davon den Mittelwert bilden. Dann kann man tiefere Frequenzen messen:
	NF Signal von 250 bis NEU 2600 Hz. (moeglicherweise muss ich das Eingangsfilter aendern, nein, das geht so.)

	3. Ausgang Si5351 soll 455kHz machen fuer 2. ZF

	TX-enable verbessert, schaltet bei unzulaessigen Frequenzen ab.
	
	Der Jumper-Engang (ADC0) wird als analoge Spannung gelesen, um 3 Zustaende zu kriegen.
		Jumper mit 1k2 (2.5Volt) => 30m Band = 10.136MHz


Feb 22	Inbetriebnahme der seriellen Schnittstelle mit CAT-Protokoll fuer Kenwood TS2000 (einzustellen im WSJTX	!)
	Das Programm auf Kenwood TS2000 eingestellt, fragt jede Sekunde
	1. "IF;"
	2. "FA;"
	3. "MD;"
	diese Werte ab.
*/

#include "si5351.h"		/* die Library fuer den Clock-Generator */

#include "Wire.h"

/*------------------------------------------------------------------------------*/


#define CAT_IF		1	/* Freigabe des CAT-Interface	*/

/*	analoge Eingaenge							*/
#define AIN0_inp	6	/* 6 = port number for PD6(AIN0) */
#define AIN1_inp	7	/* 7 = port number for PD7(AIN1) */


/*	digitale Eingaenge							*/
#define DIN0_inp	14	/* 14 = port number for PC0 */
#define DIN1_inp	15	/* 15 = port number for PC1 */


/*	digitale Ausgaenge							*/
#define TX_ENABLE_out	13	/* 13 = port number for PB5(SCK) und LED yellow !! */

#define Test1_out	12	/* 12 = port number for PB4(MISO) */
#define Test2_out	11	/* 11 = port number for PB3(MOS1) */
#define Test3_out	10	/* 10 = port number for PB2(SS) */


/*	KONSTANTE								*/
#define C_TX_40m	707400000ul	/* in Hz mal 100  = 10mHz */
#define C_TX_30m	1013600000ul	/* in Hz mal 100  = 10mHz */
#define C_TX_20m	1407400000ul	/* in Hz mal 100  = 10mHz */
#define C_ZF_FREQ	45500000ul	/* in Hz mal 100  = 10mHz */

#define C_NF_FREQ_LO	25000ul		/* =>  250Hz; (in Hz mal 100  = 10mHz) */
#define C_NF_FREQ_HI	260000ul	/* => 2600Hz; (in Hz mal 100  = 10mHz) */

#define C_TO_LIMIT	65024u		/* = 0xFE00;	Time-Out Limit */

/*------------------------------------------------------------------------------*/

Si5351	si5351c;		// create class

/*------------------------------------------------------------------------------*/
/*	List of Frequencies	*/
//TX_freq= 1840000;
//TX_freq= 3573000;
//TX_freq= 5357000;    
//TX_freq= 7038600;
//TX_freq= 7074000;
//TX_freq= 7445000;
//TX_freq= 10136000; //FT8 
//TX_freq= 10138700; //WSPR
//TX_freq= 14074000;
//TX_freq= 18100000;
//TX_freq= 21074000;
//TX_freq= 24915000;
//TX_freq= 28074000;
//TX_freq= 26048000;
//TX_freq= 50313000;

unsigned long	TX_freq;	//	= 7074000;	// in Hz

unsigned long	TX_base_freq	= C_TX_40m;	// in 10mHz

int	TxStatus	= 0;	/* 0 =  RX; 1 = TX */
int	ad_val;		/* ADC value, Jumper Eingang */


/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
/* zur CAT Communication */

int	mode		= 2;   
String	s_recvd;
String	s_recvdPart1;
String	s_recvdPart2;    
String	s_command1;
String	s_command2;  
String	s_param1;
String	s_param2; 
String	s_sent1;
String	s_sent2;

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

/* Periodenmessung */
unsigned int	d1,d2;		/* Timerwerte abspeichern */
unsigned long	sum;		/* fuer die Addition der beiden Werte */
unsigned long	codefreq;
unsigned long	timcnt;

/*------------------------------------------------------------------------------*/


/**************************************************************************/
/*
    Arduino setup function (automatically called at startup)
*/
/**************************************************************************/

void setup(void)
{

int	wert;
long	cal_factor	= 0;	/* parts per billion */

	/*--------------------------------------------*/

	Wire.setClock(500000);		/* I2C Bus Speed */

	/*--------------------------------------------*/
	/* Init serielle SST */
	/*--------------------------------------------*/

	/* default ist: SERIAL_8N1 !!  ich schreib's trotzdem hin! */
	Serial.begin(57600, SERIAL_8N1);

	Serial.setTimeout(3);	/* in milli Sek. */

//	Serial.println("Salut Le Monde!");	/* Test Ausgabe */
	/*--------------------------------------------*/

/*------------------------------------------------------------------------------*/


	/* analoger Eingang fuer Jumper */
	/* Unbedingt auf Input schaltern ! */
	pinMode(A0, INPUT);
	
	ad_val	= analogRead(A0);
	dec_jumper(ad_val);

//#ifdef DBG001
	Serial.println(ad_val);  
//#endif

/*------------------------------------------------------------------------------*/
/*------------------------------------------------------------------------------*/

	/* 0 = 25MHz; correction is parts per billion */
	cal_factor  = -33000;
	si5351c.init(SI5351_CRYSTAL_LOAD_8PF, 0, cal_factor);
 
	si5351c.set_pll(SI5351_PLL_FIXED, SI5351_PLLA);

	/*----------------------------------------------*/
	/* Clock2 Output for Receiver second ZF 455kHz	*/	
	/*----------------------------------------------*/
	si5351c.set_freq(C_ZF_FREQ, SI5351_CLK2);
	si5351c.drive_strength(SI5351_CLK2, SI5351_DRIVE_2MA);
	si5351c.output_enable(SI5351_CLK2, 1);

	/*----------------------------------------------*/
	/* Clock1 Output for Receiver			*/	
	/*----------------------------------------------*/
	si5351c.set_freq(TX_base_freq + C_ZF_FREQ, SI5351_CLK1);
	si5351c.drive_strength(SI5351_CLK1, SI5351_DRIVE_2MA);  // SI5351_DRIVE_8MA
	si5351c.output_enable(SI5351_CLK1, 1);

	/*----------------------------------------------*/
	/* Clock0 Output for Transmitter		*/	
	/*----------------------------------------------*/
	si5351c.set_freq(TX_base_freq, SI5351_CLK0);
	si5351c.drive_strength(SI5351_CLK0, SI5351_DRIVE_2MA);  // SI5351_DRIVE_8MA

	/* ACHTUNG !! jetzt noch nicht enable, sonst wird sofort gesendet */
//	si5351c.output_enable(SI5351_CLK0, 1);

/*------------------------------------------------------------------------------*/

	TCCR1A	= 0x00;		// Timer1 kein Compare Output Mode, kein Waveform Generation Mode
	TCCR1B	= 0x01;		// Timer1 0x07 = 3 Bits zur Clocksource: 0x01 = no prescaling, also 16 MHz Clock
	TCCR1B	= 0x81;		// Timer1 0x80 = setze Input Capture Noise Canceler

	/* setze Bit 2 = ACIC: Analog Comparator Input Capture Enable */
	/* im    ACSR = Analog Comparator Control and Status Register */
	ACSR	|= (1 << ACIC);

	/* Eingang der zu messenden Frequenz */
	pinMode(AIN1_inp, INPUT);	/* PD7 = AIN1 = HiZ */

	/* UNUSED	PD6 = AIN0 = 0 */

	/* Ausgang, um Sendesignal am UND Gatter freizugeben */
	pinMode(TX_ENABLE_out, OUTPUT);	/* TX-Enable output */
	digitalWrite(TX_ENABLE_out, 0);

	/* weitere Hilfsausgaenge init */
	pinMode(Test1_out, OUTPUT);
	digitalWrite(Test1_out, 0);

	pinMode(Test2_out, OUTPUT);
	digitalWrite(Test2_out, 0);

	pinMode(Test3_out, OUTPUT);
	digitalWrite(Test3_out, 0);

} /* end setup */


/*------------------------------------------------------------------------------*/
/*------------------------------------------------------------------------------*/
/*------------------------------------------------------------------------------*/

void loop(void)
{

int	FSK	= 8;


	/*------------------------------------------------------------------------------*/
	/* Modulationsfrequenz messen ueber Analog Comparator Pin7 = AN1		*/
	/*------------------------------------------------------------------------------*/

	while (FSK > 0){

#ifdef	CAT_IF
		/* CAT Interface abchecken */
		/* muss immer mitlaufen, sonst stuerzt WJSTX ab */
		if(Serial.available() > 0) cat(); 
#endif


		/* erste Vollschwingung messen	*/
		timcnt	= periode(d1);

		/* zweite Vollschwingung messen	*/
		timcnt	= periode(d2);

		sum	= d1;
		sum	= sum + d2;
#ifdef DBG001
//		Serial.print("sum=");	Serial.println(sum);
#endif

	/*------------------------------------------------------------------------------*/

		/* ist der Timer1 NICHT uebergelaufen? */
		if (timcnt != 0) {
			digitalWrite(Test1_out, 1);

			/* ja, alles OK */

			/* !! hier wird mit 32 Bit gerechnet !! */
			/* Faktor 100 liefert Pseudo-Nachkommastellen, die der Si5351 aber bestens vertraegt und braucht! */
			/* der Mittelwert von (sum = d1 + d2) / 2 ist hier eingerechnet, das 1/2 wird zum Faktor 2 fuer die 16 MHz => 32 MHz */
			codefreq	= 3200000000 / sum;	/* Frequenz aus Periodendauer berechnen, bezogen auf den Clock von 16MHz * 100; */
#ifdef DBG001
//			Serial.print(codefreq);	Serial.println(" cofr");
#endif

#ifdef DBG001
			Serial.println(codefreq);
#endif

			/* jetzt von 250.00Hz bis 2600.00Hz  */
			if ( (codefreq >= C_NF_FREQ_LO) and (codefreq < C_NF_FREQ_HI) ) {

				if (TxStatus == 0){
					digitalWrite(TX_ENABLE_out,1);
					delay(10);	/* ms */
					si5351c.output_enable(SI5351_CLK0, 1);	// TX on
					
					TxStatus = 1;
				} /* if */

				si5351c.set_freq((TX_base_freq + codefreq), SI5351_CLK0);
#ifdef DBG001
//				Serial.println("neue frq");  
#endif
			}
			else {
				/* Sendung beenden, weil eine unzulaessige NF-Frequenz erkannt wurde */
				digitalWrite(Test3_out, 1);	/* Zum Messen mit Oszi */

///				si5351c.output_enable(SI5351_CLK0, 0);		// TX off
///				digitalWrite(TX_ENABLE_out,0);  
//#ifdef DBG003
///				int hilf	= codefreq / 100;
///				Serial.println(hilf);
//#endif
				delayMicroseconds(100);		/* Zeit in us */
				digitalWrite(Test3_out, 0);
			} /* if */

			digitalWrite(Test1_out, 0);

		}
		else{

			FSK--;
#ifdef DBG001
//			Serial.println(FSK);
#endif
		} /* if */

	/*------------------------------------------------------------------------------*/

	} /* while FSK > 0 */

	/* hierher gekommen bedeutet, das NF-Signal ist weg, also "PTT losgelassen" */

	/* Sendung beenden */
	si5351c.output_enable(SI5351_CLK0, 0);   // TX off
	
//	delay(10);	/* ms */
	digitalWrite(TX_ENABLE_out,0);  
	
	TxStatus = 0;

	/* analogen Eingang fuer Jumper lesen */
	/* hier nicht mehr, sonst kann man die Frequenz nicht vom PC setzen */
//	ad_val	= analogRead(A0);
//	dec_jumper(ad_val);  


#ifdef	CAT_IF
//	if (!TxStatus) {
//		/* sind auf Empfang */
		/* CAT Interface abchecken */
		/* muss immer mitlaufen, sonst stuerzt WJSTX ab */
		if(Serial.available() > 0) cat(); 
//	} /* if */
#endif
	
} /* end loop */

/*------------------------------------------------------------------------------*/
/*------------------------------------------------------------------------------*/


/*------------------------------------------------------------------------------*/
/*	Jumper Decodieren: 0V = 20m; 2.5V = 30m; 5V = 40m - Band;		*/
/*	NANO verwendet 10 Bit ADC, also Werte von 0 bis 1023			*/
/*------------------------------------------------------------------------------*/

void	dec_jumper(int i_ad_val)
{

#ifdef DBG002	/* Absichtlich */
	Serial.println(i_ad_val);  
#endif
	if (i_ad_val < 40) {
		/* low, also zu, also mit Jumper */
		TX_base_freq		= C_TX_20m;	// in 10mHz
#ifdef DBG001
		Serial.println("low");  
#endif
	}
	else if (i_ad_val > 980) {
		/* high, also offen, also kein Jumper */
		TX_base_freq		= C_TX_40m;	// in 10mHz
#ifdef DBG001
		Serial.println("high");  
#endif
	}
	else if ( (i_ad_val > 470) && (ad_val < 550) ) {
		/* halbe Spannung, also ein Jumper mit 1k2 */
		TX_base_freq		= C_TX_30m;	// in 10mHz
#ifdef DBG001
		Serial.println("middle");  
#endif
	} /* if */

} /* end loop */

/*------------------------------------------------------------------------------*/
/*------------------------------------------------------------------------------*/


/*------------------------------------------------------------------------------*/
/*	Eine ganze Periode messen						*/
/*------------------------------------------------------------------------------*/

unsigned int	periode(unsigned int &zeit)
{
unsigned int icr;

	digitalWrite(Test2_out, 0);	// debug Port 11 = low

	TCNT1 = 0;	// Timer/Counter1 Register loeschen
	while ( (ACSR & (1 << ACO) ) == 0) {
		/* Bit 5 = ACO: Analog Comparator Output ist LOW */
		if (TCNT1 > C_TO_LIMIT) {
			break;
		}
	}
	/*---- das Signal hat gerade von 0 nach 1 gewechselt. */

	TCNT1 = 0;	// Timer/Counter1 Register loeschen
	while ( ACSR & (1 << ACO) ) {
		/* Bit 5 = ACO: Analog Comparator Output ist HIGH */
		if (TCNT1 > C_TO_LIMIT) {
			goto abbruch;
		}
	}
	/*---- das Signal hat gerade von 1 nach 0 gewechselt. */

	/*---------------------------------*/
	/* ab jetzt laeuft die Zeitmessung */
	/*---------------------------------*/

	digitalWrite(Test2_out, 1);	// debug Port = high

	TCNT1 = 0;	// Timer/Counter1 Register loeschen

	while ( (ACSR & (1 << ACO) ) == 0) {
		/* Bit 5 = ACO: Analog Comparator Output ist LOW */
		if (TCNT1 > C_TO_LIMIT) {
			goto abbruch;
		}
	}
	/*---- das Signal hat gerade von 0 nach 1 gewechselt. */
	while ( ACSR & (1 << ACO) ) {
		/* Bit 5 = ACO: Analog Comparator Output ist HIGH */
		if (TCNT1 > C_TO_LIMIT) {
			goto abbruch;
		}
	}
	/*---- das Signal hat gerade von 1 nach 0 gewechselt. */

	while ( (ACSR & (1 << ACO) ) == 0) {
		/* Bit 5 = ACO: Analog Comparator Output ist LOW */
		if (TCNT1 > C_TO_LIMIT) {
			goto abbruch;
		}
	}
	/*---- das Signal hat gerade von 0 nach 1 gewechselt. */

	while ( ACSR & (1 << ACO) ) {
		/* Bit 5 = ACO: Analog Comparator Output ist HIGH */
		if (TCNT1 > C_TO_LIMIT) {
			goto abbruch;
		}
	}
	/*---- das Signal hat gerade von 1 nach 0 gewechselt. Ein Sinus vorbei */

	/* gemessene Zeit merken */
	icr	= ICR1;		// Input Capture Register 1 abspeichern
	zeit	= icr;

	digitalWrite(Test2_out, 0);	// debug Port = low

//	while ( ACSR & (1 << ACO) ) {
//		/* Bit 5 = ACO: Analog Comparator Output ist HIGH */
//		if (TCNT1 > C_TO_LIMIT) {
//			goto abbruch;
//		}
//	}
//	/*---- das Signal hat gerade von 1 nach 0 gewechselt. Ein Sinus vorbei */

#ifdef DBG001
//	Serial.print("ICR1 ");	Serial.print(icr);	Serial.print(";   TCNT ");	Serial.println(TCNT1);
#endif

	return(TCNT1);

/*==============================================================================*/
abbruch:
	digitalWrite(Test2_out, 0);	// debug Port = low

	return(0);

} /* end periode */

/*------------------------------------------------------------------------------*/
/*------------------------------------------------------------------------------*/


/*------------------------------------------------------------------------------*/
/*	serielles CAT - Interface zum PC bedienen				*/
/*------------------------------------------------------------------------------*/

#ifdef	CAT_IF

void cat(void)
{
int	bufferIndex	= 0;
String data	= "";

	/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
	/* empfangene Daten aufbereiten */
	/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

	s_recvd	= Serial.readString();

	s_recvd.toUpperCase();
	s_recvd.replace("\n","");

	for (int i = 0; i < s_recvd.length(); ++i) {
	
		char c = s_recvd[i];
		
		if (c != ';') {		/* ';' = Command-Terminator */
			data += c;
		}
		else
		/* Command-Ende erreicht */
		if (bufferIndex == 0) {  
			data	+= '\0';
			s_recvdPart1 = data;
			bufferIndex++;
			data	= "";
		}
		else {  
			data	+= '\0';
			s_recvdPart2 = data;
			bufferIndex++;
			data	= "";
		} /* if */
	
	} /* for */
	
	s_command1	= s_recvdPart1.substring(0,2);
	s_command2	= s_recvdPart2.substring(0,2);

	s_param1	= s_recvdPart1.substring(2,s_recvdPart1.length());
	s_param2	= s_recvdPart2.substring(2,s_recvdPart2.length());


	/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

	if (s_command1 == "FA") {
		/* Reads and sets the VFO A frequency. */
	
		if (s_param1 != "") {
			/* SET die empfangene Frequenz */

			/* Frequenz von WSJTX uebernehmen */			
			TX_base_freq	= s_param1.toInt() * 100;
		} /* if */

		/* READ */
		/* Return 11 digit frequency in Hz. */
		s_sent1 = "FA" 
			+ String("00000000000").substring(0,11-(String(TX_base_freq / 100).length()))
			+ String(TX_base_freq / 100) + ";";     

	} /* if "FA" */
		
	/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

	else if (s_command1 == "PS") {
		/* Sets or reads the Power ON/ OFF status. */

		s_sent1 = "PS1;";	/* Power is On */
	}
	
	/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

	else if (s_command1 == "TX") {
		/* Sets the transceiver in TX mode. */

		s_sent1 = "TX0;";	/* Main Transmitter */
///		digitalWrite(TX_ENABLE_out,1);
///		TxStatus = 1;
       
	}
	
	/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

	else if (s_command1 == "RX") {
		/* Sets the receiver function status. */

		s_sent1 = "RX0;";	/* Main Receiver */
///		digitalWrite(TX_ENABLE_out,0);
///		TxStatus = 0;
	}

	/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

	else  if (s_command1 == "ID") {
		/* Reads the transceiver ID number. */

		s_sent1 = "ID019;";	/* = Kenwood TS2000 */
	}
	
	/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

	else if (s_command1 == "AI") {
		/* Sets or reads the Auto Information (AI) function ON/ OFF. */

		s_sent1 = "AI0;";	/* AI Off */
	}

	/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

	else if (s_command1 == "IF") {
		/* Retrieves the transceiver status. */

//#ifdef QWERTZUIOP
		if (TxStatus == 1) {  
			s_sent1 = "IF" // Return 11 digit frequency in Hz.  
			+ String("00000000000").substring(0,11-(String(TX_base_freq / 100).length()))   
			//+ String(TX_freq) + String("     ") + "+" + "0000" + "0" + "0" + "0" + "00" + "0" + String(mode) + "0" + "0" + "0" + "0" + "00" + String(" ") + ";"; 
			+ String(TX_base_freq / 100) + "00000" + "+" + "0000" + "0" + "0" + "0" + "00" + "1" + String(mode) + "0" + "0" + "0" + "0" + "000" + ";"; 
		} 
		else {  
			s_sent1 = "IF" // Return 11 digit frequency in Hz.  
			+ String("00000000000").substring(0,11-(String(TX_base_freq / 100).length()))   
			//+ String(TX_freq) + String("     ") + "+" + "0000" + "0" + "0" + "0" + "00" + "0" + String(mode) + "0" + "0" + "0" + "0" + "00" + String(" ") + ";"; 
			+ String(TX_base_freq / 100) + "00000" + "+" + "0000" + "0" + "0" + "0" + "00" + "0" + String(mode) + "0" + "0" + "0" + "0" + "000" + ";"; 
		} /* if */
//#endif
	}
	
	/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

	else if (s_command1 == "MD") {
		/* Recalls or reads the operating mode status. */

		s_sent1	= "MD2;";
	} /* if */
	
	//------------------------------------------------------------------------     
	
	if (s_command2 == "ID") {  
		/* Reads the transceiver ID number. */

		s_sent2	= "ID019;";	/* Kenwood TS2000 */
	}
	    
	//------------------------------------------------------------------------     

	if (bufferIndex == 2) {
		Serial.print(s_sent2);
	}
	else {
		Serial.print(s_sent1);
	} /* if */ 
	
	//-------------------------------------------------------------------------
	
//	if ((s_command1 == "RX") or (s_command1 = "TX")) delay(50);	/* ms */
	
	/* Clear Strings */	
	s_sent1	= String("");
	s_sent2	= String("");

} /* subroutine cat */

#endif

/*------------------------------------------------------------------------------*/
/*------------------------------------------------------------------------------*/
